@@ -486,10 +486,22 @@ print.summary.crch <- function(x, digits = max(3, getOption("digits") - 3), ...)
 
 terms.crch <- function(x, model = c("location", "scale", "full"), ...) x$terms[[match.arg(model)]]
 
+model.frame.crch <- function(formula, ...) {
+  if(!is.null(formula$model)) return(formula$model)
+  formula$terms <- formula$terms$full
+  formula$call$formula <- formula$formula <- formula(formula$terms)
+  NextMethod()
+} 
+
+model.matrix.crch <- function(object, model = c("location", "scale"), ...) {
+  model <- match.arg(model)
+  rval <- if(!is.null(object$x[[model]])) object$x[[model]]
+    else model.matrix(object$terms[[model]], model.frame(object), contrasts = object$contrasts[[model]])
+  return(rval)
+}
+
 fitted.crch <- function(object, type = c("location", "scale"), ...) object$fitted.values[[match.arg(type)]]
 
-
-
 predict.crch <- function(object, newdata = NULL,
   type = c("response", "location", "scale", "quantile"), na.action = na.pass, at = 0.5, ...)
 {
@@ -578,11 +590,11 @@ predict.crch <- function(object, newdata = NULL,
         drop(X %*% object$coefficients$location + offset[[1L]])
       },
       "scale" = {
-        exp(drop(Z %*% object$coefficients$scale + offset[[2L]]))
+        object$link$scale$linkinv(drop(Z %*% object$coefficients$scale + offset[[2L]]))
       },
       "quantile" = {
         mu <- drop(X %*% object$coefficients$location + offset[[1L]])
-        sigma <- exp(drop(Z %*% object$coefficients$scale + offset[[2L]]))
+        sigma <- object$link$scale$linkinv(drop(Z %*% object$coefficients$scale + offset[[2L]]))
         df <- object$df
         qdist(at, mu, sigma, df)       
       }
